@@ -1,5 +1,7 @@
 import requests
 import getpass
+import threading
+from .threadpool import ThreadPool
 from bs4 import BeautifulSoup
 import bs4
 import re
@@ -1037,7 +1039,16 @@ class GradescopeClient:
         if owner_id is not None:
             payload["submission[owner_id]"] = owner_id
         self.last_res = res = self.submit_form(url, url, data=payload)
-        return res.ok
+        if res.ok:
+            self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+            cdata = GS_CDATA_decoder(soup=soup)
+            jsonurl = cdata.get_gon().get("submission_json_url", None)
+            if jsonurl:
+                return (True, self.base_url + jsonurl[:-len(".json")])
+            else:
+                return (None, "Submitted but could not find the new submission ID!")
+        else:
+            return (False, "Failed to make the submission!")
             
 
     def get_bitbucket_branches(self, project_name: str):
@@ -1072,11 +1083,19 @@ class GradescopeClient:
         if res.ok:
             return json.loads(res.content)
 
-    def get_github_projects(self):
+    def get_github_projects(self, page=None, per_page=None):
         if not self.logged_in:
             print("You must be logged in!")
             return
         url = "https://www.gradescope.com/github_repositories"
+        if page or per_page:
+            url += "?"
+        if page:
+            url += f"page={page}"
+        if per_page:
+            if page:
+                url += "&"
+            url += f"per_page={per_page}"
         self.last_res = res = self.session.get(url)
         if res.ok:
             return json.loads(res.content)
@@ -1091,4 +1110,51 @@ class GradescopeClient:
             return
         self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
         cdata = GS_CDATA_decoder(soup=soup)
-        return cdata.get_gon().get("roster")
\ No newline at end of file
+        return cdata.get_gon().get("roster")
+
+    def find_github_project(self, full_name: str, repos_per_page: int=100, concurrent_requests: int=None):
+        page_id = 0
+        num_repos = 0
+        found_repo = None
+        numbers_lock = threading.Lock()
+        print("Searching for GitHub Repo...")
+
+        if concurrent_requests is None:
+            p = ThreadPool(6)
+            def conc_check(data):
+                page, num_threads = data
+                nonlocal concurrent_requests
+                res = self.get_github_projects(page=page, per_page=1)
+                if res:
+                    print(page, num_threads)
+                    with numbers_lock:
+                        if concurrent_requests < num_threads:
+                            concurrent_requests = num_threads
+            concurrent_requests = 2
+            p.map(conc_check, [(500, 5), (2000, 10), (5000, 25), (10000, 50), (20000, 75), (50000, 100)])
+            p.wait_completion_and_destroy()
+        print(concurrent_requests)
+        pool = ThreadPool(concurrent_requests)
+
+        def search_for_task(pg_id):
+            nonlocal page_id, num_repos, found_repo
+            repos = self.get_github_projects(page=page_id, per_page=repos_per_page)
+            if not repos:
+                return None
+            with numbers_lock:
+                page_id += 1
+                local_pg_id = page_id
+                num_repos += len(repos)
+                print(f"Requested {num_repos} repos.", end="\r")
+            for repo in repos:
+                if repo.get("full_name") == full_name:
+                    found_repo = repo
+                    return
+            pool.add_task(search_for_task, local_pg_id)
+
+        pool.map(search_for_task, range(concurrent_requests))
+        pool.wait_completion_and_destroy()
+        print()
+        if found_repo:
+            print("Found repo!")
+        return found_repo
\ No newline at end of file
